@@ -50,6 +50,11 @@ if sqlite3:
     sqlite3.register_adapter(decimal.Decimal, lambda v: str(v))
     sqlite3.register_converter('decimal', lambda v: decimal.Decimal(v))
 
+if psycopg2:
+    import psycopg2.extensions
+    psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
+    psycopg2.extensions.register_type(psycopg2.extensions.UNICODEARRAY)
+
 
 DATABASE_NAME = os.environ.get('PEEWEE_DATABASE', 'peewee.db')
 logger = logging.getLogger('peewee.logger')
@@ -79,8 +84,10 @@ class BaseAdapter(object):
     def get_field_types(self):
         field_types = {
             'integer': 'INTEGER',
+            'bigint': 'INTEGER',
             'float': 'REAL',
             'decimal': 'DECIMAL',
+            'double': 'REAL',
             'string': 'VARCHAR',
             'text': 'TEXT',
             'datetime': 'DATETIME',
@@ -189,6 +196,8 @@ class PostgresqlAdapter(BaseAdapter):
             'primary_key_with_sequence': 'INTEGER',
             'datetime': 'TIMESTAMP',
             'decimal': 'NUMERIC',
+            'double': 'DOUBLE PRECISION',
+            'bigint': 'BIGINT',
             'boolean': 'BOOLEAN',
             'blob': 'BYTEA',
         }
@@ -225,13 +234,20 @@ class MySQLAdapter(BaseAdapter):
     def connect(self, database, **kwargs):
         if not mysql:
             raise ImproperlyConfigured('MySQLdb must be installed on the system')
-        return mysql.connect(db=database, **kwargs)
+        conn_kwargs = {
+            'charset': 'utf8',
+            'use_unicode': True,
+        }
+        conn_kwargs.update(kwargs)
+        return mysql.connect(db=database, **conn_kwargs)
 
     def get_field_overrides(self):
         return {
             'primary_key': 'integer AUTO_INCREMENT',
             'boolean': 'bool',
             'float': 'float',
+            'double': 'double precision',
+            'bigint': 'bigint',
             'text': 'longtext',
             'decimal': 'numeric',
         }
@@ -342,7 +358,7 @@ class Database(object):
             )
 
     def field_sql(self, field):
-        return '%s %s' % (self.quote_name(field.name), field.render_field_template())
+        return '%s %s' % (self.quote_name(field.db_column), field.render_field_template())
 
     def create_table(self, model_class, safe=False):
         if model_class._meta.pk_sequence and self.adapter.sequence_support:
@@ -361,22 +377,24 @@ class Database(object):
     
     def create_index(self, model_class, field_name, unique=False):
         framing = 'CREATE %(unique)s INDEX %(index)s ON %(table)s(%(field)s);'
-
-        db_table = model_class._meta.db_table
-        index_name = self.quote_name('%s_%s' % (db_table, field_name))
         
         if field_name not in model_class._meta.fields:
             raise AttributeError(
                 'Field %s not on model %s' % (field_name, model_class)
             )
         
+        field_obj = model_class._meta.fields[field_name]
+
+        db_table = model_class._meta.db_table
+        index_name = self.quote_name('%s_%s' % (db_table, field_obj.db_column))
+        
         unique_expr = ternary(unique, 'UNIQUE', '')
         
         query = framing % {
             'unique': unique_expr,
             'index': index_name,
             'table': self.quote_name(db_table),
-            'field': self.quote_name(field_name),
+            'field': self.quote_name(field_obj.db_column),
         }
         
         self.execute(query)
@@ -400,14 +418,15 @@ class Database(object):
         field = model_class._meta.fields[field_name]
         return 'ALTER TABLE %s RENAME COLUMN %s TO %s' % (
             self.quote_name(model_class._meta.db_table),
-            self.quote_name(field_name),
+            self.quote_name(field.db_column),
             self.quote_name(new_name),
         )
     
     def drop_column_sql(self, model_class, field_name):
+        field = model_class._meta.fields[field_name]
         return 'ALTER TABLE %s DROP COLUMN %s' % (
             self.quote_name(model_class._meta.db_table),
-            self.quote_name(field_name),
+            self.quote_name(field.db_column),
         )
     
     @require_sequence_support
@@ -494,18 +513,13 @@ class MySQLDatabase(Database):
         constraint = 'fk_%s_%s_%s' % (
             db_table,
             field.to._meta.db_table,
-            field.name,
+            field.db_column,
         )
-
-        if field.name not in model_class._meta.fields:
-            raise AttributeError(
-                'Field %s not on model %s' % (field_name, model_class)
-            )
  
         query = framing % {
             'table': self.quote_name(db_table),
             'constraint': self.quote_name(constraint),
-            'field': self.quote_name(field.name),
+            'field': self.quote_name(field.db_column),
             'to': self.quote_name(field.to._meta.db_table),
             'to_field': self.quote_name(field.to._meta.pk_name),
             'cascade': ' ON DELETE CASCADE' if field.cascade else '',
@@ -518,7 +532,7 @@ class MySQLDatabase(Database):
         field = model_class._meta.fields[field_name]
         return 'ALTER TABLE %s CHANGE COLUMN %s %s %s' % (
             self.quote_name(model_class._meta.db_table),
-            self.quote_name(field_name),
+            self.quote_name(field.db_column),
             self.quote_name(new_name),
             field.render_field_template(),
         )
@@ -556,8 +570,8 @@ class QueryResultWrapper(object):
     def model_from_rowset(self, model_class, attr_dict):
         instance = model_class()
         for attr, value in attr_dict.iteritems():
-            if attr in instance._meta.fields:
-                field = instance._meta.fields[attr]
+            if attr in instance._meta.columns:
+                field = instance._meta.columns[attr]
                 setattr(instance, attr, field.python_value(value))
             else:
                 setattr(instance, attr, value)
@@ -582,20 +596,20 @@ class QueryResultWrapper(object):
                 if isinstance(col, tuple):
                     if len(col) == 3:
                         model = self.model # special-case aggregates
-                        field_name = attr = col[2]
+                        col_name = attr = col[2]
                     else:
-                        field_name, attr = col
+                        col_name, attr = col
                 else:
-                    field_name = attr = col
+                    col_name = attr = col
                 
                 if model not in collected_models:
                     collected_models[model] = model()
                 
                 instance = collected_models[model]
                 
-                if field_name in instance._meta.fields:
-                    field = instance._meta.fields[attr]
-                    setattr(instance, attr, field.python_value(value))
+                if col_name in instance._meta.columns:
+                    field = instance._meta.columns[attr]
+                    setattr(instance, field.name, field.python_value(value))
                 else:
                     setattr(instance, attr, value)
             
@@ -616,10 +630,10 @@ class QueryResultWrapper(object):
                     continue
                 
                 if not joined_inst.get_pk():
-                    joined_inst.set_pk(getattr(inst, fk_field.name))
+                    joined_inst.set_pk(getattr(inst, fk_field.id_storage))
                 
-                setattr(inst, fk_field.descriptor, joined_inst)
-                setattr(inst, fk_field.name, joined_inst.get_pk())
+                setattr(inst, fk_field.name, joined_inst)
+                setattr(inst, fk_field.id_storage, joined_inst.get_pk())
         
         return inst
     
@@ -649,22 +663,29 @@ class QueryResultWrapper(object):
                 pass
             self.__idx = idx
     
+    def iterate(self):
+        row = self.cursor.fetchone()
+        if row:
+            return self.construct_instance(row)
+        else:
+            self._populated = True
+            raise StopIteration
+    
+    def iterator(self):
+        while 1:
+            yield self.iterate()
+    
     def next(self):
         if self.__idx < self.__ct:
             inst = self._result_cache[self.__idx]
             self.__idx += 1
             return inst
         
-        row = self.cursor.fetchone()
-        if row:
-            instance = self.construct_instance(row)
-            self._result_cache.append(instance)
-            self.__ct += 1
-            self.__idx += 1
-            return instance
-        else:
-            self._populated = True
-            raise StopIteration
+        instance = self.iterate()
+        self._result_cache.append(instance)
+        self.__ct += 1
+        self.__idx += 1
+        return instance
 
 
 # create
@@ -912,15 +933,16 @@ class BaseQuery(object):
             else:
                 op = 'eq'
             
+            if lhs in model._meta.columns:
+                lhs = model._meta.columns[lhs].name
+            
             try:
                 field = model._meta.get_field_by_name(lhs)
             except AttributeError:
                 field = model._meta.get_related_field_by_name(lhs)
                 if field is None:
                     raise
-                if isinstance(rhs, Model):
-                    rhs = rhs.get_pk()
-            
+                    
             if op == 'in':
                 if isinstance(rhs, SelectQuery):
                     lookup_value = rhs
@@ -951,7 +973,7 @@ class BaseQuery(object):
                 operation = self.operations[op] % self.interpolation
             
             parsed.append(
-                (field.name, (operation, self.lookup_cast(op, lookup_value)))
+                (field.db_column, (operation, self.lookup_cast(op, lookup_value)))
             )
         
         return parsed
@@ -988,15 +1010,17 @@ class BaseQuery(object):
     def use_aliases(self):
         return len(self._joined_models) > 0 or self.force_alias
 
-    def combine_field(self, alias, field_name):
-        quoted = self.qn(field_name)
+    def combine_field(self, alias, field_col):
+        quoted = self.qn(field_col)
         if alias:
             return '%s.%s' % (alias, quoted)
         return quoted
 
     def safe_combine(self, model, alias, col):
-        if col in model._meta.fields:
+        if col in model._meta.columns:
             return self.combine_field(alias, col)
+        elif col in model._meta.fields:
+            return self.combine_field(alias, model._meta.fields[col].db_column)
         return col
     
     def follow_joins(self, current, alias_map, alias_required, alias_count, seen=None):
@@ -1018,12 +1042,12 @@ class BaseQuery(object):
             from_model = current
             field = from_model._meta.get_related_field_for_model(model, on)
             if field:
-                left_field = field.name
+                left_field = field.db_column
                 right_field = model._meta.pk_name
             else:
                 field = from_model._meta.get_reverse_related_field_for_model(model, on)
                 left_field = from_model._meta.pk_name
-                right_field = field.name
+                right_field = field.db_column
             
             if join_type is None:
                 if field.null and model not in self._where_models:
@@ -1050,7 +1074,6 @@ class BaseQuery(object):
         alias_map = {}
 
         alias_required = self.use_aliases()
-        
         if alias_required:
             alias_count += 1
             alias_map[self.model] = 't%d' % alias_count
@@ -1136,7 +1159,10 @@ class BaseQuery(object):
         return combined
 
     def convert_subquery(self, subquery):
-        subquery.query, orig_query = subquery.model._meta.pk_name, subquery.query
+        orig_query = subquery.query
+        if subquery.query == '*':
+            subquery.query = subquery.model._meta.pk_name
+        
         subquery.force_alias, orig_alias = True, subquery.force_alias
         sql, data = subquery.sql()
         subquery.query = orig_query
@@ -1258,22 +1284,22 @@ class SelectQuery(BaseQuery):
         return res.fetchone()[0]
     
     @returns_clone
-    def group_by(self, clause):
+    def group_by(self, *clauses):
         model = self.query_context
-        
-        if isinstance(clause, basestring):
-            fields = (clause,)
-        elif isinstance(clause, (list, tuple)):
-            fields = clause
-        elif issubclass(clause, Model):
-            model = clause
-            fields = clause._meta.get_field_names()
-        
-        self._group_by.append((model, fields))
+        for clause in clauses:
+            if isinstance(clause, basestring):
+                fields = (clause,)
+            elif isinstance(clause, (list, tuple)):
+                fields = clause
+            elif issubclass(clause, Model):
+                model = clause
+                fields = clause._meta.get_field_names()
+            
+            self._group_by.append((model, fields))
     
     @returns_clone
-    def having(self, clause):
-        self._having.append(clause)
+    def having(self, *clauses):
+        self._having = clauses
     
     @returns_clone
     def distinct(self):
@@ -1367,22 +1393,28 @@ class SelectQuery(BaseQuery):
                 idx = q[model].index('*')
                 q[model] =  q[model][:idx] + model._meta.get_field_names() + q[model][idx+1:]
             
-            for col in q[model]:
-                model_cols.append((model, col))
-                
-                if isinstance(col, tuple):
-                    if len(col) == 3:
-                        func, col, col_alias = col
+            for clause in q[model]:
+                if isinstance(clause, tuple):
+                    if len(clause) == 3:
+                        func, col_name, col_alias = clause
+                        column = model._meta.get_column(col_name)
                         aggregates.append('%s(%s) AS %s' % \
-                            (func, self.safe_combine(model, alias, col), col_alias)
+                            (func, self.safe_combine(model, alias, column), col_alias)
                         )
-                    elif len(col) == 2:
-                        col, col_alias = col
+                        model_cols.append((model, (func, column, col_alias)))
+                    elif len(clause) == 2:
+                        col_name, col_alias = clause
+                        column = model._meta.get_column(col_name)
                         columns.append('%s AS %s' % \
-                            (self.safe_combine(model, alias, col), col_alias)
+                            (self.safe_combine(model, alias, column), col_alias)
                         )
+                        model_cols.append((model, (column, col_alias)))
+                    else:
+                        raise ValueError('Clause must be either a 2- or 3-tuple')
                 else:
-                    columns.append(self.safe_combine(model, alias, col))
+                    column = model._meta.get_column(clause)
+                    columns.append(self.safe_combine(model, alias, column))
+                    model_cols.append((model, column))
         
         return ', '.join(columns + aggregates), model_cols
 
@@ -1407,7 +1439,7 @@ class SelectQuery(BaseQuery):
                 alias = ''
 
             for field in clause:
-                group_by.append(self.combine_field(alias, field))
+                group_by.append(self.safe_combine(model, alias, field))
 
         parsed_query, model_cols = self.parse_select_query(alias_map)
         query_meta = {
@@ -1429,11 +1461,12 @@ class SelectQuery(BaseQuery):
         order_by = []
         for piece in self._order_by:
             model, field, ordering = piece
-            if field in model._meta.fields:
-                field = self.qn(field)
-                if self.use_aliases():
-                    field = '%s.%s' % (alias_map[model], field)
-            order_by.append('%s %s' % (field, ordering))
+            if use_aliases:
+                alias = alias_map[model]
+            else:
+                alias = ''
+            
+            order_by.append('%s %s' % (self.safe_combine(model, alias, field), ordering))
         
         pieces = [select]
         
@@ -1494,6 +1527,9 @@ class UpdateQuery(BaseQuery):
     def parse_update(self):
         sets = {}
         for k, v in self.update_query.iteritems():
+            if k in self.model._meta.columns:
+                k = self.model._meta.columns[k].name
+            
             try:
                 field = self.model._meta.get_field_by_name(k)
             except AttributeError:
@@ -1504,7 +1540,7 @@ class UpdateQuery(BaseQuery):
             if not isinstance(v, F):
                 v = field.db_value(v)
             
-            sets[field.name] = v
+            sets[field.db_column] = v
         
         return sets
     
@@ -1590,8 +1626,17 @@ class InsertQuery(BaseQuery):
         cols = []
         vals = []
         for k, v in self.insert_query.iteritems():
-            field = self.model._meta.get_field_by_name(k)
-            cols.append(self.qn(k))
+            if k in self.model._meta.columns:
+                k = self.model._meta.columns[k].name
+            
+            try:
+                field = self.model._meta.get_field_by_name(k)
+            except AttributeError:
+                field = self.model._meta.get_related_field_by_name(k)
+                if field is None:
+                    raise
+            
+            cols.append(self.qn(field.db_column))
             vals.append(field.db_value(v))
         
         return cols, vals
@@ -1657,7 +1702,7 @@ def convert_lookup(model, joins, lookup):
                     if not isinstance(field, ForeignKeyField):
                         continue
                     
-                    if piece in (field.name, field.descriptor, field.related_name):
+                    if piece in (field.name, field.db_column, field.related_name):
                         joined_model = field.to
                 
                 if not joined_model:
@@ -1798,7 +1843,7 @@ class Field(object):
         return {}
     
     def __init__(self, null=False, db_index=False, unique=False, verbose_name=None,
-                 help_text=None, *args, **kwargs):
+                 help_text=None, db_column=None, *args, **kwargs):
         self.null = null
         self.db_index = db_index
         self.unique = unique
@@ -1806,6 +1851,7 @@ class Field(object):
         self.default = kwargs.get('default', None)
         self.verbose_name = verbose_name
         self.help_text = help_text
+        self.db_column = db_column
         
         kwargs['nullable'] = ternary(self.null, '', ' NOT NULL')
         self.attributes.update(kwargs)
@@ -1817,6 +1863,7 @@ class Field(object):
         self.name = name
         self.model = klass
         self.verbose_name = self.verbose_name or re.sub('_+', ' ', name).title()
+        self.db_column = self.db_column or self.name
         setattr(klass, name, FieldDescriptor(self))
     
     def get_db_field(self):
@@ -1903,6 +1950,10 @@ class IntegerField(Field):
             return int(value)
 
 
+class BigIntegerField(IntegerField):
+    db_field = 'bigint'
+
+
 class BooleanField(IntegerField):
     db_field = 'boolean'
     
@@ -1924,6 +1975,10 @@ class FloatField(Field):
             return float(value)
 
 
+class DoubleField(FloatField):
+    db_field = 'double'
+
+
 class DecimalField(Field):
     db_field = 'decimal'
     field_template = '%(column_type)s(%(max_digits)d, %(decimal_places)d)%(nullable)s'
@@ -1963,6 +2018,7 @@ class ForeignRelatedObject(object):
         self.to = to
         self.field = field
         self.field_name = self.field.name
+        self.field_column = self.field.id_storage
         self.cache_name = '_cache_%s' % self.field_name
     
     def __get__(self, instance, instance_type=None):
@@ -1970,7 +2026,7 @@ class ForeignRelatedObject(object):
             return self.field
         
         if not getattr(instance, self.cache_name, None):
-            id = getattr(instance, self.field_name, 0)
+            id = getattr(instance, self.field_column, 0)
             qr = self.to.select().where(**{self.to._meta.pk_name: id})
             try:
                 setattr(instance, self.cache_name, qr.get())
@@ -1980,9 +2036,16 @@ class ForeignRelatedObject(object):
         return getattr(instance, self.cache_name, None)
     
     def __set__(self, instance, obj):
-        assert isinstance(obj, self.to), "Cannot assign %s, invalid type" % obj
-        setattr(instance, self.field_name, obj.get_pk())
-        setattr(instance, self.cache_name, obj)
+        if self.field.null and obj is None:
+            setattr(instance, self.field_column, None)
+            setattr(instance, self.cache_name, None)
+        else:
+            if isinstance(obj, int):
+                setattr(instance, self.field_column, obj)
+            else:
+                assert isinstance(obj, self.to), "Cannot assign %s, invalid type" % obj
+                setattr(instance, self.field_column, obj.get_pk())
+                setattr(instance, self.cache_name, obj)
 
 
 class ReverseForeignRelatedObject(object):
@@ -2013,9 +2076,14 @@ class ForeignKeyField(IntegerField):
         super(ForeignKeyField, self).__init__(null=null, *args, **kwargs)
     
     def add_to_class(self, klass, name):
-        self.descriptor = name
-        self.name = name + '_id'
+        self.name = name
         self.model = klass
+        self.db_column = self.db_column or self.name + '_id'
+        
+        if self.name == self.db_column:
+            self.id_storage = self.db_column + '_id'
+        else:
+            self.id_storage = self.db_column
 
         if self.to == 'self':
             self.to = self.model
@@ -2028,8 +2096,8 @@ class ForeignKeyField(IntegerField):
             self.related_name = klass._meta.db_table + '_set'
         
         klass._meta.rel_fields[name] = self.name
-        setattr(klass, self.descriptor, ForeignRelatedObject(self.to, self))
-        setattr(klass, self.name, FieldDescriptor(self))
+        setattr(klass, self.name, ForeignRelatedObject(self.to, self))
+        setattr(klass, self.id_storage, None)
         
         reverse_rel = ReverseForeignRelatedObject(klass, self.name)
         setattr(self.to, self.related_name, reverse_rel)
@@ -2072,6 +2140,7 @@ class BaseModelOptions(object):
         self.rel_fields = {}
         self.reverse_relations = {}
         self.fields = {}
+        self.columns = {}
         self.model_class = model_class
     
     def get_sorted_fields(self):
@@ -2088,6 +2157,14 @@ class BaseModelOptions(object):
             return self.fields[name]
         raise AttributeError('Field named %s not found' % name)
     
+    def get_column_names(self):
+        return self.columns.keys()
+    
+    def get_column(self, field_or_col):
+        if field_or_col in self.fields:
+            return self.fields[field_or_col].db_column
+        return field_or_col
+    
     def get_related_field_by_name(self, name):
         if name in self.rel_fields:
             return self.fields[self.rel_fields[name]]
@@ -2095,13 +2172,13 @@ class BaseModelOptions(object):
     def get_related_field_for_model(self, model, name=None):
         for field in self.fields.values():
             if isinstance(field, ForeignKeyField) and field.to == model:
-                if name is None or name == field.name or name == field.descriptor:
+                if name is None or name == field.name or name == field.db_column:
                     return field
     
     def get_reverse_related_field_for_model(self, model, name=None):
         for field in model._meta.fields.values():
             if isinstance(field, ForeignKeyField) and field.to == self.model_class:
-                if name is None or name == field.name or name == field.descriptor:
+                if name is None or name == field.name or name == field.db_column:
                     return field
     
     def rel_exists(self, model):
@@ -2137,8 +2214,6 @@ class BaseModel(type):
                             continue
                         if field_name in cls.__dict__:
                             continue
-                        if isinstance(field_obj, ForeignKeyField):
-                            field_name = field_obj.descriptor
                         field_copy = copy.deepcopy(field_obj)
                         setattr(cls, field_name, field_copy)
 
@@ -2160,6 +2235,7 @@ class BaseModel(type):
             if isinstance(attr, Field):
                 attr.add_to_class(cls, name)
                 _meta.fields[attr.name] = attr
+                _meta.columns[attr.db_column] = attr
                 if isinstance(attr, PrimaryKeyField):
                     _meta.pk_name = attr.name
         
@@ -2192,28 +2268,35 @@ class Model(object):
     __metaclass__ = BaseModel
     
     def __init__(self, *args, **kwargs):
-        self.get_field_dict()
+        self.initialize_defaults()
+        
         for k, v in kwargs.items():
             setattr(self, k, v)
     
+    def initialize_defaults(self):
+        for field in self._meta.fields.values():
+            if field.default is not None:
+                if callable(field.default):
+                    field_value = field.default()
+                else:
+                    field_value = field.default
+                setattr(self, field.name, field_value)
+    
     def __eq__(self, other):
         return other.__class__ == self.__class__ and \
                self.get_pk() and \
                other.get_pk() == self.get_pk()
     
     def get_field_dict(self):
-        def get_field_val(field):
-            field_value = getattr(self, field.name)
-            if not self.get_pk() and field_value is None and field.default is not None:
-                if callable(field.default):
-                    field_value = field.default()
-                else:
-                    field_value = field.default
-                setattr(self, field.name, field_value)
-            return (field.name, field_value)
+        field_dict = {}
         
-        pairs = map(get_field_val, self._meta.fields.values())
-        return dict(pairs)
+        for field in self._meta.fields.values():
+            if isinstance(field, ForeignKeyField):
+                field_dict[field.name] = getattr(self, field.id_storage)
+            else:
+                field_dict[field.name] = getattr(self, field.name)
+        
+        return field_dict
     
     @classmethod
     def table_exists(cls):
